/*
 * Copyright (C) 2003      Martyn Russell <ginxd@btopenworld.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * $RCSfile: gj_gtk_progress.c,v $
 *
 * $Author: mr $
 * $Date: 2003/12/07 17:58:52 $
 *
 * $Revision: 1.1.1.1 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/* gui */
#include <glade/glade.h>
#include <gtk/gtk.h>

/* local */
#include "common.h"

#include "gj_main.h"
#include "gj_support.h"

#include "gj_gtk_progress.h"
#include "gj_gtk_support.h"


static gboolean gj_gtk_progress_cancel(GjProgressDialog *dialog);
static gboolean gj_gtk_progress_setup(GjProgressDialog *dialog);

/* internal callbacks */
static gboolean gj_gtk_progress_pulse_cb(GjProgressDialog *dialog);

/*
 * actual GUI functions
 */
static void on_destroy(GtkWidget *widget, GjProgressDialog *dialog);
static void on_response(GtkDialog *widget, gint response, GjProgressDialog *dialog);


struct t_GjProgressDialog
{
  void *user_data;

  gint pulse_id;

  /* properties */
  gchar *primary;
  gchar *title;
  gchar *secondary; 

  gboolean modal;
  GtkWindow *transient_window;

  GjProgressDialogCallback cb;

  gint response;

  /* controls */
  GtkWidget *window;

  GtkWidget *button_cancel;

  GtkWidget *label_primary;
  GtkWidget *label_secondary;

  GtkWidget *progressbar;
};


/*
 * actual function to modify the prg gui  
 */
GjProgressDialog *gj_gtk_progress_load(gboolean modal, 
				       GtkWindow *transient_window,
				       GjProgressDialogCallback cb, 
				       gpointer user_data,
				       const gchar *primary, 
				       const gchar *secondary, ...)
{
  GladeXML *xml = NULL;
  GjProgressDialog *dialog = NULL;

  va_list args;
  gchar *buffer = NULL;

  va_start(args,secondary);
  buffer = g_strdup_vprintf(secondary,args);
  va_end(args);

  /* check vars */
  if(primary == NULL)
    {
      g_warning("%s: primary was NULL", __FUNCTION__);
      return NULL;
    }
  
  if(cb == NULL)
    {
      g_warning("%s: callback was NULL", __FUNCTION__);
      return NULL;
    }
  
  /* create new struct */
  dialog = g_new0(struct t_GjProgressDialog, 1);
  
  dialog->user_data = user_data;
  
  dialog->pulse_id = -1;

  dialog->primary = g_strdup(primary);
  dialog->secondary = buffer;
  
  dialog->transient_window = transient_window;
  
  dialog->modal = modal;
  dialog->cb = cb;
 
  xml = gj_gtk_glade_get_file(gnome_jabber_get_glade_file(),
			      "progress",
			      NULL,
			      "progress", &dialog->window,
			      "button_cancel", &dialog->button_cancel,
			      "label_primary", &dialog->label_primary,
			      "label_secondary", &dialog->label_secondary,
			      "progressbar", &dialog->progressbar,
			      NULL);

  gj_gtk_glade_connect(xml,
		       dialog,
		       "progress", "response", on_response,
		       "progress", "destroy", on_destroy,
		       NULL);

  if(!xml) return FALSE;
  g_object_unref(xml);

  gj_gtk_progress_setup(dialog);

  return dialog;
}

void gj_gtk_progress_unload(GjProgressDialog *dialog)
{
  if(dialog == NULL)
    {
      g_warning("%s: window was NULL", __FUNCTION__);
      return;
    }

  if(dialog->cb != NULL)
    {
      g_message("%s: using callback:0x%.8x, with user_data:0x%.8x", 
		__FUNCTION__, (gint)dialog->cb, (gint)dialog->user_data);

      /* call callback function */
      (dialog->cb)(FALSE, dialog->user_data);
    }
  
  gtk_widget_destroy(dialog->window);
}

static gboolean gj_gtk_progress_setup(GjProgressDialog *dialog)
{
  gchar *text = NULL;

  if(dialog == NULL)
    {
      g_warning("%s: GjProgressDialog *was NULL", __FUNCTION__);
      return FALSE;
    }

  g_message("%s: using user_data: 0x%.8x", __FUNCTION__, (gint)dialog->user_data);
  g_message("%s: using callback: 0x%.8x", __FUNCTION__, (gint)dialog->cb);

  /* set text */
  g_message("%s: setting primary...", __FUNCTION__);
  text = g_strdup_printf("<span size=\"11000\" weight=\"heavy\">%s</span>", dialog->primary);
  gtk_label_set_markup(GTK_LABEL(dialog->label_primary), text);
  g_free(text);

  if(dialog->secondary != NULL)
    {
      g_message("%s: setting secondary...", __FUNCTION__);
      text = g_strdup_printf("%s", dialog->secondary);
      gtk_label_set_markup(GTK_LABEL(dialog->label_secondary), text);
      g_free(text);
    }

  /* set modal prodialog */
  g_message("%s: setting window to %s", __FUNCTION__, dialog->modal?"be modal":"NOT be modal");
  gtk_window_set_modal(GTK_WINDOW(dialog->window), dialog->modal);


  /* set transient window */
  if(dialog->transient_window != NULL)
    {
      g_message("%s: setting transient window: 0x%.8x", __FUNCTION__, 
		(gint)dialog->transient_window);
      gtk_window_set_transient_for(GTK_WINDOW(dialog->window), 
				   GTK_WINDOW(dialog->transient_window));
    }

  /* set timeout */
  dialog->pulse_id = g_timeout_add(50, (GSourceFunc)gj_gtk_progress_pulse_cb, dialog);
  
  gdk_window_set_decorations (GDK_WINDOW(dialog->window->window), 
			      GDK_DECOR_BORDER | GDK_DECOR_TITLE);

  return TRUE;
}

static gboolean gj_gtk_progress_cancel(GjProgressDialog *dialog)
{
  if(dialog == NULL)
    {
      g_warning("%s: GjProgressDialog *was NULL", __FUNCTION__);
      return FALSE;
    }

  if(dialog->cb != NULL)
    {
      g_message("%s: using callback:0x%.8x, with user_data:0x%.8x", 
		__FUNCTION__, (gint)dialog->cb, (gint)dialog->user_data);

      /* call callback function */
      (dialog->cb)(TRUE, dialog->user_data);
    }
     
  return TRUE;
}

static gboolean gj_gtk_progress_pulse_cb(GjProgressDialog *dialog)
{
  if(!dialog)
    return FALSE;
  
  gtk_progress_bar_pulse(GTK_PROGRESS_BAR(dialog->progressbar));
  return TRUE;
}

/*
 * actual gui functions
 */
static void on_destroy(GtkWidget *widget, GjProgressDialog *dialog)
{
  if(!dialog)
    return;

  if(dialog->primary != NULL)
    {
      g_free(dialog->primary);
      dialog->primary = NULL;
    }

  if(dialog->secondary != NULL)
    {
      g_free(dialog->secondary);
      dialog->secondary = NULL;
    }

  if(dialog->pulse_id > 0)
    {
      g_source_remove(dialog->pulse_id);
      dialog->pulse_id = -1;
    }

  g_free(dialog);
}

static void on_response(GtkDialog *widget, gint response, GjProgressDialog *dialog)
{
  if(!dialog)
    return;

  switch(response) 
    {
    case GTK_RESPONSE_DELETE_EVENT:
    case GTK_RESPONSE_CANCEL:
      gj_gtk_progress_cancel(dialog);
      break;
    }

  gtk_widget_destroy(dialog->window);
}


#ifdef __cplusplus
}
#endif
